/***************************************************************************
 $RCSfile$
                             -------------------
    cvs         : $Id: banking.cpp 935 2006-02-14 02:11:55Z aquamaniac $
    begin       : Mon Mar 01 2004
    copyright   : (C) 2004 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "cppgui_p.h"
#include <assert.h>

#include <gwenhywfar/inherit.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/gui_be.h>



GWEN_INHERIT(GWEN_GUI, CppGui);




int CppGuiLinker::MessageBox(GWEN_GUI *gui,
			     uint32_t flags,
			     const char *title,
			     const char *text,
			     const char *b1,
			     const char *b2,
			     const char *b3,
			     uint32_t guiid){
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->messageBox(flags, title, text, b1, b2, b3, guiid);
}



int CppGuiLinker::InputBox(GWEN_GUI *gui,
			   uint32_t flags,
			   const char *title,
			   const char *text,
			   char *buffer,
			   int minLen,
			   int maxLen,
			   uint32_t guiid){
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->inputBox(flags, title, text, buffer, minLen, maxLen, guiid);
}



uint32_t CppGuiLinker::ShowBox(GWEN_GUI *gui,
			       uint32_t flags,
			       const char *title,
			       const char *text,
			       uint32_t guiid){
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->showBox(flags, title, text, guiid);
}



void CppGuiLinker::HideBox(GWEN_GUI *gui, uint32_t id){
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->hideBox(id);
}



uint32_t CppGuiLinker::ProgressStart(GWEN_GUI *gui,
				     uint32_t flags,
				     const char *title,
				     const char *text,
				     uint64_t total,
				     uint32_t guiid){
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->progressStart(flags, title, text, total, guiid);
}



int CppGuiLinker::ProgressAdvance(GWEN_GUI *gui,
				  uint32_t id,
				  uint64_t progress){
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->progressAdvance(id, progress);
}



int CppGuiLinker::ProgressLog(GWEN_GUI *gui,
			      uint32_t id,
			      GWEN_LOGGER_LEVEL level,
			      const char *text){
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->progressLog(id, level, text);
}



int CppGuiLinker::ProgressEnd(GWEN_GUI *gui, uint32_t id){
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->progressEnd(id);
}



int CppGuiLinker::Print(GWEN_GUI *gui,
			const char *docTitle,
			const char *docType,
			const char *descr,
			const char *text,
			uint32_t guiid){
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->print(docTitle, docType, descr, text, guiid);
}



int CppGuiLinker::GetPassword(GWEN_GUI *gui,
			      uint32_t flags,
			      const char *token,
			      const char *title,
			      const char *text,
			      char *buffer,
			      int minLen,
			      int maxLen,
			      uint32_t guiid) {
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->getPassword(flags, token, title, text, buffer, minLen, maxLen, guiid);
}



int CppGuiLinker::SetPasswordStatus(GWEN_GUI *gui,
				    const char *token,
				    const char *pin,
				    GWEN_GUI_PASSWORD_STATUS status,
				    uint32_t guiid) {
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->setPasswordStatus(token, pin, status, guiid);
}



int CppGuiLinker::CheckCert(GWEN_GUI *gui,
			    const GWEN_SSLCERTDESCR *cert,
			    GWEN_IO_LAYER *io,
			    uint32_t guiid) {
  CppGui *xgui;

  assert(gui);
  xgui=GWEN_INHERIT_GETDATA(GWEN_GUI, CppGui, gui);
  assert(xgui);

  return xgui->checkCert(cert, io, guiid);
}



GWENHYWFAR_CB
void CppGuiLinker::freeData(void *bp, void *p) {
  CppGui *xgui;

  DBG_NOTICE(0, "CppGuiLinker: Freeing CppGui");
  xgui=(CppGui*)p;
  if (xgui->_gui) {
    xgui->_gui=0;
  }
  delete xgui;
}












CppGui::CppGui()
:_gui(NULL) {
  _gui=GWEN_Gui_new();
  GWEN_INHERIT_SETDATA(GWEN_GUI, CppGui,
		       _gui, this,
		       CppGuiLinker::freeData);
  GWEN_Gui_SetMessageBoxFn(_gui, CppGuiLinker::MessageBox);
  GWEN_Gui_SetInputBoxFn(_gui, CppGuiLinker::InputBox);
  GWEN_Gui_SetShowBoxFn(_gui, CppGuiLinker::ShowBox);
  GWEN_Gui_SetHideBoxFn(_gui, CppGuiLinker::HideBox);
  GWEN_Gui_SetProgressStartFn(_gui, CppGuiLinker::ProgressStart);
  GWEN_Gui_SetProgressAdvanceFn(_gui, CppGuiLinker::ProgressAdvance);
  GWEN_Gui_SetProgressLogFn(_gui, CppGuiLinker::ProgressLog);
  GWEN_Gui_SetProgressEndFn(_gui, CppGuiLinker::ProgressEnd);
  GWEN_Gui_SetPrintFn(_gui, CppGuiLinker::Print);
  GWEN_Gui_SetGetPasswordFn(_gui, CppGuiLinker::GetPassword);
  GWEN_Gui_SetSetPasswordStatusFn(_gui, CppGuiLinker::SetPasswordStatus);
  GWEN_Gui_SetCheckCertFn(_gui, CppGuiLinker::CheckCert);
}



CppGui::~CppGui(){
  DBG_NOTICE(0, "~CppGui: Freeing CppGui");
  if (_gui) {
    GWEN_INHERIT_UNLINK(GWEN_GUI, CppGui, _gui)
    GWEN_Gui_free(_gui);
  }
}



int CppGui::messageBox(uint32_t flags,
		       const char *title,
		       const char *text,
		       const char *b1,
		       const char *b2,
		       const char *b3,
		       uint32_t guiid){
  return GWEN_ERROR_NOT_SUPPORTED;
}



int CppGui::inputBox(uint32_t flags,
		     const char *title,
		     const char *text,
		     char *buffer,
		     int minLen,
		     int maxLen,
		     uint32_t guiid){
  return GWEN_ERROR_NOT_SUPPORTED;
}



uint32_t CppGui::showBox(uint32_t flags,
			 const char *title,
			 const char *text,
			 uint32_t guiid){
  return 0;
}



void CppGui::hideBox(uint32_t id){
}



uint32_t CppGui::progressStart(uint32_t flags,
			       const char *title,
			       const char *text,
			       uint64_t total,
			       uint32_t guiid){
  return 0;
}



int CppGui::progressAdvance(uint32_t id, uint64_t progress){
  return GWEN_ERROR_NOT_SUPPORTED;
}



int CppGui::progressLog(uint32_t id,
			GWEN_LOGGER_LEVEL level,
			const char *text){
  return GWEN_ERROR_NOT_SUPPORTED;
}



int CppGui::progressEnd(uint32_t id){
  return GWEN_ERROR_NOT_SUPPORTED;
}



int CppGui::print(const char *docTitle,
		  const char *docType,
		  const char *descr,
		  const char *text,
		  uint32_t guiid){
  return GWEN_ERROR_NOT_SUPPORTED;
}



int CppGui::getPassword(uint32_t flags,
			const char *token,
			const char *title,
			const char *text,
			char *buffer,
			int minLen,
			int maxLen,
			uint32_t guiid) {
  return GWEN_ERROR_NOT_SUPPORTED;
}



int CppGui::checkCert(const GWEN_SSLCERTDESCR *cert,
		      GWEN_IO_LAYER *io,
		      uint32_t guiid) {
  return GWEN_ERROR_NOT_SUPPORTED;
}



int CppGui::setPasswordStatus(const char *token,
			      const char *pin,
			      GWEN_GUI_PASSWORD_STATUS status,
			      uint32_t guiid) {
  return GWEN_ERROR_NOT_SUPPORTED;
}



GWEN_GUI *CppGui::getCInterface(){
  return _gui;
}





